from google_nest_client.device import Device


class Camera(Device):
    def get_last_motion_event(self) -> dict:
        return self.get_event('CameraMotion.Motion')

    def get_last_person_event(self) -> dict:
        return self.get_event('CameraPerson.Person')

    def get_last_sound_event(self) -> dict:
        return self.get_event('CameraSound.Sound')

    def generate_image(self, event_id: str) -> dict:
        return self.api_client.execute_command(
            self.device_id,
            'sdm.devices.commands.CameraEventImage.GenerateImage',
            {'eventId': event_id},
        )



from google_nest_client.device import Device


class Thermostat(Device):
    def get_hvac_status(self) -> str:
        return self.get_trait('ThermostatHvac')['status']

    def get_mode(self) -> str:
        return self.get_trait('ThermostatMode')['mode']

    def get_ambient_temperature(self) -> float:
        deg_c = self.get_trait('Temperature')['ambientTemperatureCelsius']
        return celsius_to_farenheit(deg_c)

    def get_heating_temperature(self) -> float:
        deg_c = self.get_trait('ThermostatTemperatureSetpoint')['heatCelsius']
        return celsius_to_farenheit(deg_c)

    def get_ambient_humidity(self) -> float:
        return self.get_trait('Humidity')['ambientHumidityPercent']

    def set_heat(self, deg_f: float) -> None:
        self.api_client.execute_command(
            self.device_id,
            'sdm.devices.commands.ThermostatTemperatureSetpoint.SetHeat',
            {'heatCelsius': farenheit_to_celsius(deg_f)},
        )


def farenheit_to_celsius(f):
    return (f - 32) * 5 / 9


def celsius_to_farenheit(c):
    return c * 9 / 5 + 32



from typing import Any


class Device:
    def __init__(self, api_client: Any, device_dict: dict):
        self.api_client = api_client
        self.device_dict = device_dict
        self.device_id = get_device_id(device_dict)

    def label(self) -> str:
        return get_device_label(self.device_dict)

    def get_trait(self, trait: str) -> dict:
        return self.device_dict['traits']['sdm.devices.traits.' + trait]

    def get_event(self, trait: str) -> dict:
        return self.device_dict['traits']['sdm.devices.events.' + trait]


def get_device_id(device_dict: dict) -> str:
    return device_dict['name'].split('/')[-1]


def get_device_label(device_dict: dict) -> str:
    label = device_dict.get('traits', {}).get('sdm.devices.traits.Info', {}).get('customName')
    if label:
        return label

    return device_dict.get('parentRelations', {})[0].get('displayName')



from typing import List

from google_nest_client.client import GoogleNestClient
from google_nest_client.device_cache import DeviceCache


class GoogleNestCachingClient(GoogleNestClient):
    def __init__(
        self,
        project_id: str,
        device_cache: DeviceCache,
        cache_writes_enabled: bool = True,
    ):
        super().__init__(project_id)
        self.device_cache = device_cache
        self.cache_writes_enabled = cache_writes_enabled

        if self.cache_writes_enabled:
            self.device_cache.clear()
            self.load_cache()

    def load_cache(self) -> None:
        for device in super().get_devices():
            self.device_cache.upsert_device(device)

    def update_from_event(self, event: dict) -> None:
        if 'resourceUpdate' in event:
            self.device_cache.upsert_device(event['resourceUpdate'])

    def get_devices(self) -> List[dict]:
        return self.device_cache.get_devices()

    def get_device(self, device_id) -> dict:
        return self.device_cache.get_device(device_id)



from typing import List
from typing import Optional

from google_nest_client.api_client import GoogleNestAPIClient
from google_nest_client.camera import Camera
from google_nest_client.thermostat import Thermostat


class GoogleNestClient(GoogleNestAPIClient):
    def get_cameras(self) -> List[Camera]:
        return [
            Camera(self, device)
            for device in self.get_devices_by_type('sdm.devices.types.CAMERA')
        ]

    def get_thermostats(self) -> List[Thermostat]:
        return [
            Thermostat(self, device)
            for device in self.get_devices_by_type('sdm.devices.types.THERMOSTAT')
        ]

    def get_camera(self, device_id: str) -> Camera:
        return Camera(self, self.get_device(device_id))

    def get_thermostat(self, device_id: Optional[str] = None) -> Thermostat:
        if device_id:
            return Thermostat(self, self.get_device(device_id))
        else:
            device_dicts = self.get_devices_by_type('sdm.devices.types.THERMOSTAT')
            return Thermostat(self, device_dicts[0])

    def get_camera_by_label(self, label: str) -> Optional[Camera]:
        matched_devices = self.get_devices_by_type_and_label(
            'sdm.devices.types.CAMERA',
            label,
        )

        if matched_devices:
            return Camera(self, matched_devices[0])
        else:
            return None

    def get_thermostat_by_label(self, label: str = 'Thermostat') -> Optional[Thermostat]:
        matched_devices = self.get_devices_by_type_and_label(
            'sdm.devices.types.THERMOSTAT',
            label,
        )

        if matched_devices:
            return Thermostat(self, matched_devices[0])
        else:
            return None



from typing import Iterable

import time
from abc import ABCMeta
from abc import abstractmethod

from google_nest_client.device import get_device_id


class DeviceCache(metaclass=ABCMeta):
    @abstractmethod
    def clear(self) -> None:
        pass

    @abstractmethod
    def upsert_device(self, device_update: dict) -> None:
        pass

    def get_devices(self) -> Iterable[dict]:
        pass

    def get_device(self, device_id: str) -> dict:
        pass


class InMemoryDeviceCache(DeviceCache):
    def __init__(self) -> None:
        self.id_to_device: dict = {}

    def clear(self) -> None:
        self.id_to_device = {}

    def upsert_device(self, device_update: dict) -> None:
        device_id = get_device_id(device_update)
        device = self.id_to_device.get(device_id)

        if not device:
            self.id_to_device[device_id] = device_update
            return

        if 'traits' in device_update:
            for k, v in device_update['traits']:
                device['traits'][k] = v

        if 'events' in device_update:
            for k, v in device_update['events']:
                device['events'][k] = v
                device['events'][k]['timestamp'] = int(time.time())

    def get_devices(self) -> Iterable[dict]:
        return self.id_to_device.values()

    def get_device(self, device_id: str) -> dict:
        return self.id_to_device[device_id]



from typing import List
from typing import Optional

import requests
from abc import ABCMeta
from abc import abstractmethod

from google_nest_client.device import get_device_label


API_URI = 'https://smartdevicemanagement.googleapis.com/v1/enterprises/'


class AuthenticationError(Exception):
    pass


class GoogleNestAPIClient(metaclass=ABCMeta):
    def __init__(self, project_id: str) -> None:
        self.project_id = project_id

    @abstractmethod
    def get_access_token(self) -> str:
        pass

    def api_get(self, endpoint: str) -> dict:
        resp = requests.get(
            API_URI + self.project_id + endpoint,
            headers={
                'Authorization': 'Bearer ' + self.get_access_token(),
                'Content-type': 'application/json',
            },
        )

        try:
            resp.raise_for_status()
        except requests.HTTPError as ex:
            if ex.response.status_code in (401, 403):
                raise AuthenticationError

        return resp.json()

    def api_post(self, endpoint: str, json: Optional[dict] = None) -> dict:
        resp = requests.post(
            API_URI + self.project_id + endpoint,
            headers={
                'Authorization': 'Bearer ' + self.get_access_token(),
                'Content-type': 'application/json',
            },
            json=json,
        )

        try:
            resp.raise_for_status()
        except requests.HTTPError as ex:
            if ex.response.status_code in (401, 403):
                raise AuthenticationError

        return resp.json()

    def get_structures(self) -> dict:
        return self.api_get(
            '/structures'
        )['structures']

    def get_devices(self) -> List[dict]:
        return self.api_get(
            '/devices'
        )['devices']

    def get_device(self, device_id: str) -> dict:
        return self.api_get('/devices/' + device_id)

    def get_devices_by_type(self, device_type):
        return [
            device for device in self.get_devices()
            if device['type'] == device_type
        ]

    def get_devices_by_type_and_label(
        self,
        device_type: str,
        label: str
    ) -> List[dict]:
        return [
            device for device in self.get_devices_by_type(device_type)
            if get_device_label(device) == label
        ]

    def execute_command(
        self,
        device_id: str,
        command: str,
        params: str
    ) -> dict:
        return self.api_post(
            '/devices/' + device_id + ':executeCommand',
            json={
                'command': command,
                'params': params,
            },
        )



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='google_nest_client',
    version='1.0.0',
    description='Google Nest API Client',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='',
    author='Jimming Cheng',
    author_email='jimming@gmail.com',
    packages=['google_nest_client'],
    install_requires=[
        'requests',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
